Scriptname zMFqCageManagerScript extends Quest  
;=============================
;PROPERTIES
;=============================

zMFqMainScript property qMain Auto
{Set zMFqMain}

ReferenceAlias[] Property CaptiveList  Auto  
{Alias Ref List for Captives}

ObjectReference Property CageMarker  Auto  
{Place of Captive}

MiscObject Property moCaptured  Auto
{String Object : " is captured."}  

GlobalVariable Property zMFgAllow0xFF  Auto  
{This variable allow to capture temporary ref}

Message Property MsgFull  Auto  
Message Property MsgTempActor  Auto  

;=============================
;FUNCTIONS
;=============================
int Function FindEmptyCage()
	return qMain.FindEmptyAlias(CaptiveList)
EndFunction

;Captive in/out---------------------------------------------------------------------------

Function CaptureByBall(Actor akTarget)
	if akTarget.IsDead() || akTarget.IsDeleted()
		return
	endif

	if !Game.GetPlayer().IsDetectedBy(akTarget) || akTarget.GetActorValuePercentage("Health") < 0.5 || qMain.GetTrained(akTarget) >=100
		RemoveInvalidRefFromCage()
		if CaptivePutInCage(akTarget)
			if akTarget.IsInCombat()
				akTarget.StopCombat()
			endif
			akTarget.AllowPCDialogue(true)
		endif
	endif
EndFunction

Bool Function CaptivePutInCage(ObjectReference o)
	int i =  FindEmptyCage()
	if i < 0
		MsgFull.show()
		return false

	elseif Math.LogicalAnd(o.GetFormID(), 0xFF000000) == 0xFF000000 && zMFgAllow0xFF.GetValue() == 0
		MsgTempActor.Show()
		return false

	else
		Debug.Notification( (o as Actor).GetLeveledActorBase().getName() + moCaptured.getName())
		ReferenceAlias temp = CaptiveList[i]
		temp.ForceRefTo(o)
		o.MoveTo(CageMarker)
		return true
	endif
EndFunction


Function CaptivePutOutTo(int index, ObjectReference Target = NONE, ReferenceAlias NextAlias = NONE)
	ReferenceAlias temp = CaptiveList[index]
	ObjectReference Captive = temp.GetRef()

	if !Target
		Target = Game.GetPlayer()
	endif

	if  qMain.IsEmptyRef(Captive)
		qMain.ClearAliasAndName(temp)
		return
	else
		Captive.MoveTo(Target) ;; They must be Loaded Cell when Swapping Alias
		if NextAlias
			NextAlias.ForceRefTo(Captive)
		endif
		qMain.ClearAliasAndName(temp)
		return
	endif
EndFunction

ObjectReference Function GetCaptiveRef(int index)
	ReferenceAlias temp = CaptiveList[index]
	return temp.GetRef()
EndFunction

Function ClearCage(int index)
	ReferenceAlias temp = CaptiveList[index]
	qMain.ClearAliasAndName(temp)
EndFunction

Function InitAllCage()
	qMain.InitAlisaList(CaptiveList)
EndFunction

Function RemoveInvalidRefFromCage()
	qMain.RemoveInvalidRefFromAliasList(CaptiveList)
EndFunction

Function ReleaseAllCaptives()
	int index = CaptiveList.Length - 1
	ReferenceAlias tmpAlias
	ObjectReference tmpObj
	While index >= 0
		tmpAlias = CaptiveList[index]
		tmpObj = tmpAlias.GetRef()
		if !qMain.IsEmptyRef(tmpObj)
			tmpObj.MoveToMyEditorLocation()
			qMain.ClearAliasAndName(tmpAlias)
		endif
		index -=1
	endWhile
EndFunction

;-------------------------------------------------------------
;UI Extension
;-------------------------------------------------------------
int Function ShowCageListUI()
	UIListMenu myList = UIExtensions.GetMenu("UIListMenu") as UIListMenu
	myList.ResetMenu()
	int Max = CaptiveList.Length - 1
	int index = 0
	Actor Captive
	String sName

	While index <= Max
		Captive = GetCaptiveRef(index) as Actor
  		if qMain.IsEmptyRef(Captive)
			sName = "-----"
		else
			sName = Captive.GetLeveledActorBase().GetName()
		endif
		index += 1
		myList.AddEntryItem( index + ":" + sName)
	endWhile
	myList.AddEntryItem("Quit")
	myList.OpenMenu()
	return myList.GetResultInt()
EndFunction

Function OpenReleaseCattleUI()
	int Max = CaptiveList.Length ; +1 is "Quit" Entry
	int b = ShowCageListUI()
	if b < 0 || b >= Max
		return
	endif
	while b >= 0 && b < Max 
		CaptivePutOutTo(b)
		utility.wait(0.1)
		b = ShowCageListUI()
	endwhile
EndFunction

