Scriptname zMFqFarmManagerScript extends Quest  

;=============================
;PROPERTIES
;=============================
zMFqMainScript property qMain Auto
{set zMFqMain}

zMFqCageManagerScript Property qCageManager Auto
{set  zMFqCageManager}

FormList Property MarkerListX  Auto  
{Position Markers for StockListX}

FormList Property MarkerListY  Auto 
{Position Markers for StockListY} 

ReferenceAlias[] Property StockListX Auto
{Alias List - X01 X02 X03 ...}

ReferenceAlias[] Property StockListY Auto
{Alias List - Y01 Y02 Y03 ...}

ObjectReference Property HarvestChest  Auto  
{Container Ref for Harvest}

GlobalVariable Property gFundPoint Auto  
{Fund point for the farm.}

GlobalVariable Property gBreedTimescale Auto  
{timescale for breeding}


;=============================
;Variables
;=============================
Bool bUpdate = False

;=============================
;Event
;=============================
Event OnUpdateGameTime() 	
	if !bUpdate || gBreedTimescale.GetValue() < 24.0 || GetFarmPoint()<=0
		StopBreeding()
		return
	else
		qMain.RemoveInvalidRefFromAliasList(StockListX)
		qMain.RemoveInvalidRefFromAliasList(StockListY)
		if qMain.IsEmptyAliasList(StockListX) || qMain.IsEmptyAliasList(StockListY)
			StopBreeding()
			return
		else
			DoBreeding()
			return
		endif
	endif
endEvent

;=============================
;FUNCTIONS
;=============================
;---------------------------------------------------------
;Breeding
;---------------------------------------------------------
Function StartBreeding()
	if bUpdate || gBreedTimescale.GetValue() < 24.0 || GetFarmPoint()<=0
		return

	elseif   qMain.IsEmptyAliasList(StockListX) || qMain.IsEmptyAliasList(StockListY)
		return
	else
		;Debug.Notification("Start Breeding")
		bUpdate = True
		RegisterForUpdateGameTime(gBreedTimescale.GetValue())
		return
	endif
EndFunction

Function StopBreeding()
	;Debug.Notification("Stop Breeding")
	bUpdate = False
	UnregisterForUpdateGameTime()
EndFunction

Function DoBreeding()
	int xMax = StockListX.Length - 1
	int yMax = StockListY.Length - 1
	int index
	if xMax > yMax
		index = yMax
	elseif xMax <= yMax
		index = xMax
	endif

	while index>=0
		qMain.AddHarvestTo(HarvestChest, (GetStockRef(index, False) as Actor), (GetStockRef(index, True) as Actor))
		index -=1
	endwhile

	AddProfitToSouls()
EndFunction

;---------------------------------------------------------
;Cattle Manager
;---------------------------------------------------------

Bool Function CattleHasPartner(int StockID, bool XY)
	ObjectReference Partner =  GetStockRef(StockID, !XY)
	if !qMain.IsEmptyRef(Partner)
		Return True
	else
		Return False
	endif
EndFunction

Form[] Function GetMarkerListZ(Bool XY=False)
	if XY == False
		Return MarkerListX.ToArray()
	else
		Return MarkerListY.ToArray()
	endif
EndFunction

ReferenceAlias[] Function GetStockListZ(Bool XY=False)
	if XY == False
		return StockListX
	else
		return StockListY
	endif
EndFunction

Bool Function OpenPutInFarmMenu(int StockID, Bool XY = False)
	int Max = qCageManager.CaptiveList.Length ; +1 is "Quit" Entry
	int b = qCageManager.ShowCageListUI()

	if b < 0 || b >= Max
		return false
	endif

	if PutInFarm(b, StockID, XY)
		StartBreeding()
		return True
	else
		return false
	endif
EndFunction

Bool Function PutInFarm(int captiveID, int StockID, bool XY = False) ;X=False, Y=True
	Form[] MarkerListZ = GetMarkerListZ(XY)
	ReferenceAlias[] StockZ = GetStockListZ(XY)
	ObjectReference TargetMarker = MarkerListZ[StockID] as ObjectReference
	ReferenceAlias TargetAlias = StockZ[StockID]

	if !qMain.IsEmptyRef(TargetAlias.GetRef())
		return false
	else
		qCageManager.CaptivePutOutTo(captiveID, TargetMarker, TargetAlias)
		return true
	endif

EndFunction

Bool Function BackToCage(int StockID, bool XY = False)
	ReferenceAlias[] StockZ = GetStockListZ(XY)
	ReferenceAlias TargetAlias = StockZ[StockID]

	if qMain.IsEmptyRef(TargetAlias.GetRef())
		qMain.ClearAliasAndName(TargetAlias)
		if qMain.IsEmptyAliasList(StockZ)
			StopBreeding()
		endif
		return false

	elseif qCageManager.CaptivePutInCage(TargetAlias.GetRef()) == True
		qMain.ClearAliasAndName(TargetAlias)
		if qMain.IsEmptyAliasList(StockZ)
			StopBreeding()
		endif
		return true
	else
		return false
	endif
EndFunction

ObjectReference Function GetStockRef(int StockID, bool XY = False)
	ReferenceAlias[] StockZ = GetStockListZ(XY)
	ReferenceAlias TargetAlias = StockZ[StockID]

	if qMain.IsEmptyRef(TargetAlias.GetRef())
		return NONE
	else
		return TargetAlias.GetRef()
	endif
EndFunction

;---------------------------------------------------------
;Cleaning
;---------------------------------------------------------

Function InitStockZ(bool XY=False)
	qMain.InitAlisaList(GetStockListZ(XY))
EndFunction

Function RemoveInvalidRefFromStockZ(bool XY=False)
	qMain.RemoveInvalidRefFromAliasList(GetStockListZ(XY))
EndFunction

;---------------------------------------------------------
;Menu/Account
;---------------------------------------------------------

Int Function GetFarmPoint()
	Return gFundPoint.GetValueInt()
EndFunction

Function ModFarmPoint(int i)
	gFundPoint.Mod(i)
	int cp = gFundPoint.GetValueInt()

	if cp > 1000000
		gFundPoint.SetValueInt(1000000)	
		return

	elseif cp < 0
		gFundPoint.SetValueInt(0)
		return
	endif
EndFunction

int Function GetCaptiveLimit()
	Return (qCageManager.CaptiveList.Length - 1)
EndFunction

;MENU --------------------------------------------------------------------------------------------------
int Function ShowAccountMenuWheel()

	UIWheelMenu myWheel = UIExtensions.GetMenu("UIWheelMenu") as UIWheelMenu
	myWheel.ResetMenu()

	;0 Account
	myWheel.SetPropertyIndexString("optionText", 0, qMain.mABOpiton01.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 0, qMain.mABOpiton01.GetName())
	myWheel.SetPropertyIndexInt("optionTextColor", 0, 0xFF8856)
	myWheel.SetPropertyIndexBool("optionEnabled", 0, True)

	;1 Soul Fund
	myWheel.SetPropertyIndexString("optionText", 1, qMain.mABOpiton02.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 1, qMain.mABOpiton02.GetName())
	myWheel.SetPropertyIndexInt("optionTextColor", 1, 0xFFFF99)
	myWheel.SetPropertyIndexBool("optionEnabled", 1, True)

	;2 Soul Convert
	myWheel.SetPropertyIndexString("optionText", 2, qMain.mABOpiton03.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 2, qMain.mABOpiton03.GetName())
	myWheel.SetPropertyIndexInt("optionTextColor", 2, 0x99FFFF)
	myWheel.SetPropertyIndexBool("optionEnabled", 2, True)

	;4 Time Scale
	myWheel.SetPropertyIndexString("optionText", 4, qMain.mABOpiton04.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 4, qMain.mABOpiton04.GetName())
	myWheel.SetPropertyIndexInt("optionTextColor", 4, 0xFF99FF)
	myWheel.SetPropertyIndexBool("optionEnabled", 4, True)

	;5 0xFF
	myWheel.SetPropertyIndexString("optionText", 5, qMain.mABOpiton05.GetName() + "["+ qMain.zMFgAllow0xFF.GetValueInt()+"]")
	myWheel.SetPropertyIndexString("optionLabelText", 5,  qMain.mABOpiton05.GetName() + "["+ qMain.zMFgAllow0xFF.GetValueInt()+"]")
	myWheel.SetPropertyIndexInt("optionTextColor", 5, 0x993300)
	myWheel.SetPropertyIndexBool("optionEnabled", 5, True)

	;6 Exit
	myWheel.SetPropertyIndexString("optionText", 6, "Exit")
	myWheel.SetPropertyIndexString("optionLabelText", 6, "Exit")
	myWheel.SetPropertyIndexInt("optionTextColor", 6, 0xFFFFFF)
	myWheel.SetPropertyIndexBool("optionEnabled", 6, True)

	int i = myWheel.OpenMenu()
	return i
EndFunction

;Account-------------------------------------------------------------------------------------
Int[] Function GetFarmAccount()
	int[] TotalX = qMain.GetAccount(StockListX)
	int[] TotalY = qMain.GetAccount(StockListY)
	int index = TotalX.Length - 1

	While index >= 0
		TotalX[index] = TotalX[index]+TotalY[index]
		index -=1
	EndWhile

	Return TotalX
EndFunction

Function AddProfitToSouls()
	int[] TotalX = qMain.GetAccount(StockListX)
	int[] TotalY = qMain.GetAccount(StockListY)
	int profit = TotalX[2]+TotalY[2]
	ModFarmPoint(profit)
EndFunction

;Soul Funds Charge-------------------------------------------------------------------------------------
Function SoulFundCharge()
		int b = qMain.MsgSoulFundMenu.Show(Game.GetPlayer().GetItemCount(qMain.Gold001),gFundPoint.GetValue())
		While b>=0 && b <= 2
			if b==0
				ConvertGoldToMoney(100, 50)
			elseif b==1
				ConvertGoldToMoney(500, 300)
			elseif b==2
				ConvertGoldToMoney(1000, 650)
			endif
			utility.wait(0.01)
			b = qMain.MsgSoulFundMenu.Show(Game.GetPlayer().GetItemCount(qMain.Gold001),gFundPoint.GetValue())
		endwhile
EndFunction

Function ConvertGoldToMoney(int i, int rate)
	if Game.GetPlayer().GetItemCount(qMain.Gold001) < i
		qMain.MsgGoldShortage.Show()
		return
	endif
	Game.GetPlayer().RemoveItem(qMain.Gold001, i)
	gFundPoint.Mod(rate)
EndFunction

;Soul Convert To Perk-------------------------------------------------------------------------------------
Function SoulConvert()
		int reqSouls = 3000
		int SoulFund = GetFarmPoint()
		int b = -1
		b = qMain.MsgSoulPerkMenu.Show(SoulFund, Game.GetPerkPoints(), reqSouls)
		while b== 0
			if SoulFund >= reqSouls
				ModFarmPoint(- reqSouls)
				Game.AddPerkPoints(1)
			else
				qMain.MsgSoulShortage.Show()
			endif
			utility.wait(0.01)
			SoulFund = GetFarmPoint()
			b = qMain.MsgSoulPerkMenu.Show(SoulFund, Game.GetPerkPoints(), reqSouls)
		endwhile
EndFunction

;Time Scale Menu -------------------------------------------------------------------------------------
Function ChangeBreedingTimeScale()
		Bool bLoop = False
		Int PreTime = gBreedTimescale.GetValueInt()

		bLoop = ShowMenuTimescale()
		while bLoop
			Utility.wait(0.01)
			bLoop = ShowMenuTimescale()
		endwhile

		;Reset
		if Pretime != gBreedTimescale.GetValueInt()
			StopBreeding()
			Utility.wait(0.01)
			StartBreeding()
		endif
EndFunction

Float Function GetFarmTimescale(float FarmHour)
	return FarmHour/24.0
EndFunction

Bool Function ShowMenuTimescale()
		int b = -1
		float timescale = GetFarmTimescale(gBreedTimescale.GetValue())
		b = qMain.MsgTimeScaleMenu.show(timescale)
		if b == 0 
			gBreedTimescale.Mod(24.0)
			return True
		elseif b == 1
			gBreedTimescale.Mod(-24.0)
			if gBreedTimescale.GetValue() < 24.0
				gBreedTimescale.SetValue(0.0)
			endif
			return True
		else
			return false
		endif
EndFunction

;Switch Allow FF---------------------------------------------------------------------------------------
Function SwitchAllowCapture0XFF()
	int i = qMain.zMFgAllow0xFF.GetValueInt()
	if  i == 0
		qMain.zMFgAllow0xFF.SetValueInt(1)
	else
		qMain.zMFgAllow0xFF.SetValueInt(0)
	endif
EndFunction

;---------------------------------------------------------
;Avoid Respawning
;---------------------------------------------------------
Function EvacuateFromReset(ObjectReference Shelter, bool XY=False)
	ReferenceAlias[] StockZ = GetStockListZ(XY)
	ReferenceAlias TargetAlias
	int index = StockZ.Length - 1

	while index >=0
		TargetAlias = StockZ[Index]
		if !qMain.IsEmptyRef(TargetAlias.GetRef())
			TargetAlias.TryToMoveTo(Shelter)
		endif
		index -=1
	endwhile
EndFunction

Function ReturnToFarm(bool XY=False)
	ReferenceAlias[] StockZ = GetStockListZ(XY)
	ReferenceAlias TargetAlias
	int index = StockZ.Length - 1

	while index >=0
		TargetAlias = StockZ[Index]
		if !qMain.IsEmptyRef(TargetAlias.GetRef())
			TargetAlias.GetActorRef().MoveToPackageLocation()
		endif
		index -=1
	endwhile
EndFunction

Function CattleBecomeFriend(Actor a)
	if qMain.IsEmptyRef(a)
		return
	endif
	a.IgnoreFriendlyHits()
	if a.GetAV("Aggression")>1
		a.SetAV("Aggression", 1)
	endif
	a.SetRelationshipRank(Game.GetPlayer(), 3)
EndFunction

Function CattlePoistionReset(Actor a)
	if qMain.IsEmptyRef(a)
		return
	endif
	Utility.wait(1.0)
	a.MoveToPackageLocation()
EndFunction

;---------------------------------------------------------
;UI Extension
;---------------------------------------------------------
Int Function OpenCattleStatusMenu(Actor a, Actor b = None)
	Int[] StatusArray = qMain.GetStatusStock(a,b)
	UIListMenu myList = UIExtensions.GetMenu("UIListMenu") as UIListMenu
	myList.ResetMenu()
	myList.AddEntryItem( a.GetLeveledActorBase().GetName())
	myList.AddEntryItem( qMain.moStatus01.GetName() + " : " + StatusArray[0])
	myList.AddEntryItem( qMain.moStatus02.GetName() + " : " + StatusArray[1])
	myList.AddEntryItem( qMain.moStatus03.GetName() + " : " + StatusArray[2])
	myList.AddEntryItem( qMain.moStatus04.GetName() + " : " + StatusArray[3])
	myList.AddEntryItem( qMain.moStatus05.GetName() + " : " + StatusArray[4])
	myList.AddEntryItem( qMain.moStatus06.GetName() + " : " + StatusArray[5])
	myList.AddEntryItem( qMain.moCattleRank.GetName() + " : " + GetRankOfCattle(StatusArray[0], StatusArray[1]))
	myList.OpenMenu()

	return myList.GetResultInt()
EndFunction

String Function GetRankOfCattle(int cost, int product)
	if cost <= 0
		Return "F-"
	endif
	float p = ( product as float)/ (cost as float)
	if p< 0.4
		Return "F-"
	elseif p < 0.50
		Return "F"
	elseif p < 0.55
		Return "F+"
	elseif p < 0.60
		Return "D-"
	elseif p < 0.7
		Return "D"
	elseif p < 0.8
		Return "D+"
	elseif p < 0.9
		Return "C-"
	elseif p < 1.0
		Return "C"
	elseif p <1.1
		Return "C+"
	elseif p< 1.25
		Return "B-"
	elseif p< 1.4
		Return "B"
	elseif p< 1.55
		Return "B+"
	elseif p< 1.7
		Return "A-"
	elseif p< 1.85
		Return "A"
	elseif p< 2.0
		Return "A+"
	elseif p< 2.5
		Return "S"
	elseif p< 3.0
		Return "SS"
	else
		Return "SSS"
	endif
EndFunction

int Function ShowCattleMenuWheel(Form akForm = None, Form akReceiver = None)
	if akForm == None
		return -1
	endif

	bool bHog = SetHog()
	bool bHogMaster = qMain.IsHogMaster(akForm as Actor)

	UIWheelMenu myWheel = UIExtensions.GetMenu("UIWheelMenu") as UIWheelMenu
	myWheel.ResetMenu()
	myWheel.SetPropertyIndexString("optionText", 0, qMain.moBackToCage.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 0, qMain.moBackToCage.GetName())
	if !bHogMaster
		myWheel.SetPropertyIndexInt("optionTextColor", 0, 0xFF8856)
		myWheel.SetPropertyIndexBool("optionEnabled", 0, True)
	else
		myWheel.SetPropertyIndexInt("optionTextColor", 0, 0x000000)
		myWheel.SetPropertyIndexBool("optionEnabled", 0, False)
	endif

	myWheel.SetPropertyIndexString("optionText", 1, qMain.moInventory.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 1, qMain.moInventory.GetName())
	myWheel.SetPropertyIndexInt("optionTextColor", 1, 0xFFFF99)
	myWheel.SetPropertyIndexBool("optionEnabled", 1, True)

	myWheel.SetPropertyIndexString("optionText", 2, qMain.moCattleRank.GetName())
	myWheel.SetPropertyIndexString("optionLabelText", 2, qMain.moCattleRank.GetName())
	myWheel.SetPropertyIndexInt("optionTextColor", 2, 0x99FFFF)
	myWheel.SetPropertyIndexBool("optionEnabled", 2, True)

	myWheel.SetPropertyIndexString("optionText", 3, "Exit")
	myWheel.SetPropertyIndexString("optionLabelText", 3, "Exit")
	myWheel.SetPropertyIndexInt("optionTextColor", 3, 0xFFFFFF)
	myWheel.SetPropertyIndexBool("optionEnabled", 3, True)


	if bHog

		;2p
		if IsHog(akForm as Actor)
			myWheel.SetPropertyIndexString("optionText", 4, "Stop")
			myWheel.SetPropertyIndexString("optionLabelText", 4, "Stop")
		else
			myWheel.SetPropertyIndexString("optionText", 4, qMain.moBreeding2P.GetName())
			myWheel.SetPropertyIndexString("optionLabelText", 4, qMain.moBreeding2P.GetName())
		endif
		myWheel.SetPropertyIndexInt("optionTextColor", 4, 0xFF99FF)
		myWheel.SetPropertyIndexBool("optionEnabled", 4, True)

		if bHogMaster
			int nHog = qMain.CountLinkedHogs(akForm as ObjectReference) - 1
			if nHog >= 1 && !IsHog(akForm as Actor) ;3p
				myWheel.SetPropertyIndexString("optionText", 5, qMain.moBreeding3P.GetName())
				myWheel.SetPropertyIndexString("optionLabelText", 5, qMain.moBreeding3P.GetName())
				myWheel.SetPropertyIndexInt("optionTextColor", 5, 0xFF99FF)
				myWheel.SetPropertyIndexBool("optionEnabled", 5, True)
			endif

			if nHog >= 2 && !IsHog(akForm as Actor)  ;4p
				myWheel.SetPropertyIndexString("optionText", 6, qMain.moBreeding4P.GetName())
				myWheel.SetPropertyIndexString("optionLabelText", 6, qMain.moBreeding4P.GetName())
				myWheel.SetPropertyIndexInt("optionTextColor", 6, 0xFF99FF)
				myWheel.SetPropertyIndexBool("optionEnabled", 6, True)
			endif

			if nHog >= 3 && !IsHog(akForm as Actor) ;5p
				myWheel.SetPropertyIndexString("optionText", 7, qMain.moBreeding5P.GetName())
				myWheel.SetPropertyIndexString("optionLabelText", 7, qMain.moBreeding5P.GetName())
				myWheel.SetPropertyIndexInt("optionTextColor", 7, 0xFF99FF)
				myWheel.SetPropertyIndexBool("optionEnabled", 7, True)
			endif

		elseif akForm.HasKeywordString("ActorTypeNPC")
			;Dress Up Lovers.esp
			if Game.GetModbyName("Dress Up Lovers.esp") < 255
				myWheel.SetPropertyIndexString("optionText", 5, "Dress Up Lovers")
				myWheel.SetPropertyIndexString("optionLabelText", 5, "Dress Up Lovers")
				myWheel.SetPropertyIndexInt("optionTextColor", 5, 0xFFCC00)
				myWheel.SetPropertyIndexBool("optionEnabled", 5, True)
			endif

		endif
	endif

	int i = myWheel.OpenMenu(akForm, akReceiver)
	return i
EndFunction

Function OpenCattleMenu(Int StockID, Bool XY = False)
	ObjectReference StockRef = GetStockRef(StockID, XY)
	int b = ShowCattleMenuWheel(StockRef)
	if b== 0
		BackToCage(StockID, XY)
	elseif b==1
		(StockRef as Actor).OpenInventory(TRUE)
	elseif b == 2
		OpenCattleStatusMenu((StockRef as Actor), (GetStockRef(StockID, !XY)  as Actor))

	elseif b ==3 || b == -1

	elseif b == 4 ;2p
		if !IsHog(StockRef as Actor)
			StartHog(StockID, XY)
		else
			ForceStopHog(StockID, XY)
		endif

	elseif b >=5
		if qMain.IsHogMaster(StockRef as Actor)
			if !IsHog(StockRef as Actor)
				StartHogParty(StockID, (b - 2),  XY)
			else
				ForceStopHog(StockID, XY)
			endif

		elseif b == 5
			;Dress Up Lovers.esp
			if StockRef.HasKeywordString("ActorTypeNPC")
			if Game.GetModbyName("Dress Up Lovers.esp") < 255
				Game.ForceFirstPerson()
				Utility.wait(1.0)
				Spell OutfitSpell = Game.GetFormFromFile(0x000411FE,"Dress Up Lovers.esp") as Spell
				OutfitSpell.cast(Game.GetPlayer(), StockRef)
			endif
			endif

		endif
	endif
EndFunction

;---------------------------------------------------------
;Hog Mode
;---------------------------------------------------------

Bool Function SetHog()
	return False
EndFunction

Bool Function IsHog(Actor a)
	Return False
EndFunction

Function ForceStopHog(int index, bool XY = False)
EndFunction

Function StartHog(int index, bool XY = False)
EndFunction

Function StartHogParty(int index, int num = 3, bool XY = True)
EndFunction


